@@ -2,14 +2,14 @@
 # -*- coding: utf-8 -*-
 
 import os
-import subprocess
 import logging
 
-from urllib2 import build_opener
+from urllib2 import HTTPError
 import re
 
+import fileutils
 from framework import Command, Stop
-from env import JenkinsParser
+from env import FileUtilsParser, JenkinsParser
 
 try:
     from xml.etree.ElementTree import XML
@@ -18,22 +18,16 @@ except ImportError:
 
 log = logging.getLogger("omego.artifacts")
 
-# create an opener that will simulate a browser user-agent
-opener = build_opener()
-if 'USER_AGENT' in os.environ:
-    opener.addheaders = [('User-agent', os.environ.get('USER_AGENT'))]
 
+class ArtifactException(Exception):
 
-def download(url, filename):
-    response = opener.open(url)
-    try:
-        output = open(filename, 'w')
-        try:
-            output.write(response.read())
-        finally:
-            output.close()
-    finally:
-        response.close()
+    def __init__(self, msg, path):
+        super(ArtifactException, self).__init__(msg)
+        self.path = path
+
+    def __str__(self):
+        return '%s\npath: %s' % (
+            super(ArtifactException, self).__str__(), self.path)
 
 
 class Artifacts(object):
@@ -41,23 +35,19 @@ class Artifacts(object):
     def __init__(self, args):
 
         self.args = args
-        url = opener.open(args.build+"api/xml")
-        try:
-            log.debug('Fetching xml from %s code:%d', url.url, url.code)
-            if url.code != 200:
-                log.error('Failed to get Hudson XML from %s (code %d)',
-                          url.url, url.code)
-                raise Stop(20, 'Job lookup failed, is the job name correct?')
-            hudson_xml = url.read()
-        finally:
-            url.close()
+        buildurl = args.build
 
-        root = XML(hudson_xml)
+        root = self.read_xml(buildurl)
+        if root.tag == "matrixBuild":
+            runs = root.findall("./run/url")
+            buildurl = self.find_label_matches(r.text for r in runs)
+            root = self.read_xml(buildurl)
 
         artifacts = root.findall("./artifact")
-        base_url = args.build+"artifact/"
+        base_url = buildurl + "artifact/"
         if len(artifacts) <= 0:
-            raise AttributeError("No artifacts, please check build on Hudson.")
+            raise AttributeError(
+                "No artifacts, please check build on the CI server.")
 
         patterns = self.get_artifacts_list()
         for artifact in artifacts:
@@ -69,6 +59,55 @@ class Artifacts(object):
                     setattr(self, key, rel_path)
                     pass
 
+    def read_xml(self, buildurl):
+        url = None
+        try:
+            url = fileutils.open_url(buildurl + 'api/xml')
+            log.debug('Fetching xml from %s code:%d', url.url, url.code)
+            if url.code != 200:
+                log.error('Failed to get CI XML from %s (code %d)',
+                          url.url, url.code)
+                raise Stop(20, 'Job lookup failed, is the job name correct?')
+            ci_xml = url.read()
+        except HTTPError as e:
+            log.error('Failed to get CI XML (%s)', e)
+            raise Stop(20, 'Job lookup failed, is the job name correct?')
+        finally:
+            if url:
+                url.close()
+
+        root = XML(ci_xml)
+        return root
+
+    def find_label_matches(self, urls):
+        required = set(self.args.labels.split(','))
+        if '' in required:
+            required.remove('')
+        log.debug('Searching for matrix runs matching: %s', required)
+        matches = []
+        for url in urls:
+            url_labels = self.label_list_parser(url)
+            if len(required.intersection(url_labels)) == len(required):
+                matches.append(url)
+
+        if len(matches) != 1:
+            log.error('Found %d matching matrix build runs: %s',
+                      len(matches), matches)
+            raise Stop(
+                30, 'Expected one matching run, found %d' % len(matches))
+        return matches[0]
+
+    def label_list_parser(self, url):
+        """
+        Extracts comma separate tag=value pairs from a string
+        Assumes all characters other than / and , are valid
+        """
+        labels = re.findall('([^/,]+=[^/,]+)', url)
+        slabels = set(labels)
+        if '' in slabels:
+            slabels.remove('')
+        return slabels
+
     @classmethod
     def get_artifacts_list(self):
         return {'server': r'OMERO\.server.*\.zip',
@@ -81,7 +120,6 @@ class Artifacts(object):
                 }
 
     def download(self, component):
-
         if not hasattr(self, component) or getattr(self, component) is None:
             raise Exception("No %s found" % component)
 
@@ -89,30 +127,34 @@ class Artifacts(object):
         filename = os.path.basename(componenturl)
         unzipped = filename.replace(".zip", "")
 
+        if self.args.dry_run:
+            return
+
         if os.path.exists(unzipped):
             return unzipped
 
-        if not os.path.exists(filename):
-            log.info("Downloading %s", componenturl)
-            download(componenturl, filename)
+        log.info("Checking %s", componenturl)
+        progress = 0
+        if self.args.verbose:
+            progress = 20
+        ptype, localpath = fileutils.get_as_local_path(
+            componenturl, self.args.overwrite, progress=progress,
+            httpuser=self.args.httpuser, httppassword=self.args.httppassword)
+        if ptype != 'file' or not localpath.endswith('.zip'):
+            raise ArtifactException('Expected local zip file', localpath)
 
         if not self.args.skipunzip:
-            command = [self.args.unzip]
-            if self.args.unzipargs:
-                command.append(self.args.unzipargs)
-            if self.args.unzipdir:
-                command.extend(["-d", self.args.unzipdir])
-            command.append(filename)
-            log.debug("Calling %s", command)
-            p = subprocess.Popen(command)
-            rc = p.wait()
-            if rc != 0:
-                log.error('Unzip failed')
-                raise Stop(rc, 'Unzip failed, unzip manually and run again')
-            else:
+            try:
+                log.info('Unzipping %s', localpath)
+                unzipped = fileutils.unzip(
+                    localpath, match_dir=True, destdir=self.args.unzipdir)
                 return unzipped
-        else:
-            return filename
+            except Exception as e:
+                log.error('Unzip failed: %s', e)
+                print e
+                raise Stop(20, 'Unzip failed, try unzipping manually')
+
+        return localpath
 
 
 class DownloadCommand(Command):
@@ -132,6 +174,7 @@ class DownloadCommand(Command):
             help="The artifact to download from the CI server")
 
         self.parser = JenkinsParser(self.parser)
+        self.parser = FileUtilsParser(self.parser)
 
     def __call__(self, args):
         super(DownloadCommand, self).__call__(args)
@@ -152,8 +195,5 @@ class DownloadCommand(Command):
                 log.debug("% 20s => %s" % (dest, replacement))
                 setattr(args, dest, replacement)
 
-        if args.dry_run:
-            return
-
         artifacts = Artifacts(args)
         artifacts.download(args.artifact)
